/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

#include "Statistic.h"
#include <stdio.h>
#include "Rogus.h"
#include "time.h"
#include <string.h>

static time_t startTime;
static const char* parserName = "unknown";

Statistic* Statistic::first = NULL;

static const char* myAsctime(struct tm* t)
{
	static char buf[80];
	strcpy(buf,asctime(t));
	for (unsigned int i = 0; i < strlen(buf); i++)
		if (buf[i] < ' ')
			buf[i] = 0;
	return buf;
}

void Statistic::reset(const char* s)
{
	for (Statistic* p = first; p != NULL; p = p->next)
		p->clear();
	startTime = time(NULL);
	parserName = s;
}

void Statistic::report()
{
	if (!statistics_on)
		return;
	FILE* f = fopen("Rogus.log", "a");
	fprintf(f,"Rogus McBogus %s running %s\r",rogus_version,parserName);
	struct tm* t = localtime(&startTime);
	fprintf(f,"%s to ",myAsctime(t));
	time_t stopTime = time(NULL);
	t = localtime(&stopTime);
	fprintf(f,"%s\r\r",myAsctime(t));
	for (Statistic* p = first; p != NULL; p = p->next)
		p->stat(f);
	for (int i = 0; i < 78; i++)
		fprintf(f,"-");
	fprintf(f,"\r");
	fclose(f);
}

Statistic::Statistic()
{
	prev = NULL;
	next = first;
	first = this;
	if (next != NULL)
		next->prev = this;
}

Statistic::~Statistic()
{
	if (next != NULL)
	next->prev = prev;
	if (prev != NULL)
		prev->next = next;
	else
		first = next;
}

// Author: Patrick Pelletier
