/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

#ifndef SCHED_H
#define SCHED_H

#include "data_structs.h"
#include "OMS_app.h"
#include "MIDI_lib.h"
#include "BinStatistic.h"

typedef long ScheduleTime;

typedef long (*my_wake_funcp)( void* );

class task_data
{
public:
	enum { fa_t, m_t, on_t } ty;
	ScheduleTime nwu; // 1
	// I had to de-unionify these PP 12/17/95
	struct { my_wake_funcp func; void *arg; } fa;
	MIDI_msg m;
	//unsigned long task_num; // used only in p_queue


	task_data() {}

	task_data( my_wake_funcp f, void *a, ScheduleTime t ) { ty=fa_t; fa.func=f; fa.arg=a; nwu=t; }

	task_data( const MIDI_msg& in_m, ScheduleTime t ) { ty = m_t; m = in_m; nwu=t; }

	task_data( const MIDI_msg& on, ScheduleTime dur, ScheduleTime t ) { ty = on_t; m = on; m.time = dur; nwu=t; }
};
/* 1. nwu is my copy of nextWakeUp.  A copy is necessary because when the WakeUpTask is called,
nextWakeUp is set to NEVER by OMS, and we need to use nextWakeup in the WakeUpTask. */

struct task
{
	MMRESULT ott;
	task* chain;
	task_data td;
};

class task_pool : public pool<task>
{
	BinStatistic lateness;
	static int binner(int i) { return (i*5); }
public:
	const int size;
	len_monitor mon;

	 task_pool(int s) : mon( s, "task pool" ), size(s), pool<task>(s, "" ), lateness(12, binner, "Scheduler Lateness") { }
	~task_pool() {  }

	void init( );
		
	task* store( const task_data& );

	void record_lateness( long l ) { lateness.throwInBin((int)l); }
};

//--------------------------------------------------------------------------------------------------


/* 1. The curious reader may wonder why tnp is separate from task_pool.  The reason is that if
the OMS scheduler is not working well, it will be easy to replace tnp with p_queue, a scheduler
written by me that runs at loop-time. */

class scheduler // 1
{
	// scheduler is written with the assumption that only one scheduler will exist at once,
	// so it's okay to use static members
	static task_pool tp;
	static task* currentTask;

	static err_ind do_insert(task* t);

	static void reorderTasks();
	static void waitAndHandle();
	static void removeAndExecute();
	static unsigned WINAPI threadMain(LPVOID lpvThreadParm);

	static HANDLE threadHandle;
	static UINT threadID;
	static HANDLE eventHandle;

public:
	scheduler();
	~scheduler();

	void init( OMS_app * = NULL ) { tp.init(); }

	err_ind play_note( const MIDI_msg& on, ScheduleTime dur );
	
	err_ind insert( const task_data& td )
	{
		task *t = tp.store( td );
		err_ind result = err;
		if (t != NULL)
			result = do_insert(t);
		return result;
	}

	// this version is like above but the user can get the task back so they can cancel it if they want
	err_ind insert( const task_data& td, task** task_return )
	{
		task *t = tp.store( td );
		*task_return = t;
		err_ind result = err;
		if (t != NULL)
			result = do_insert(t);
		return result;
	}

	err_ind cancel(task* t);

	void processQueue();
};

extern scheduler sched;

#endif
// Authors: Ben Denckla, Patrick Pelletier

