/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

#include "Recorder.h"
#include "err_and_warn.h"
#include "Score.h"

Recorder::Recorder()
{
	first = new storage_group;
	last = first;
	groups_free = 1;
	current = first;
	current_base_index = current_index = 0;
	overflow = 0;
	EventRecord garbage;
	SpendTime(garbage); // set up our initial buffers
	StartIdling(); // make sure buffers expand when needed
}

Recorder::~Recorder()
{
	StopIdling();
}

void Recorder::SpendTime(const EventRecord&)
{
	// we make sure there are plenty of buffers and allocate them at loop time,
	// so they will be there when we need them at interrupt time
	if (groups_free < desired_free)
	{
		int n = desired_free - groups_free;
		for (int i = 0; i < n; i++)
		{
			last->next = new storage_group;
			last = last->next;
			groups_free++;
		}
		if (overflow > 0)
		{
			current_base_index = current_index;
			current = current->next;
			ew.w("Recorder overflow -- %d messages lost",overflow);
			overflow = 0;
		}
	}
}

void Recorder::insert(const MIDI_msg* m)
{
	if (overflow > 0) // no space for event
	{
		overflow++;
		return;
	}
	index(current_index) = *m;
	int i = current_index;
	while (i > 0 && index(i-1).time > index(i).time) // in case not in chronolocal order
	{
		MIDI_msg tmp = index(i);
		index(i-1) = index(i);
		index(i--) = tmp;
	}
	current_index++;
	if (current_index-current_base_index >= storage_group::group_size)
	{
		if (groups_free > 0)
		{
			groups_free--;
			current_base_index = current_index;
			current = current->next;
		}
		else
			overflow = 1;
	}
}

void Recorder::setAutoRecord(bool)
{
	ew.w("setAutoRecord not yet implemented");
}

bool Recorder::getAutoRecord()
{
	ew.w("getAutoRecord not yet implemented");
	return 0;
}

/*
FiledataPtr Recorder::capture()
{
	int n = oap->howManyOutputNodes();
	MidiMsgArrayPtrArrayPtr mmapap = new MidiMsgArrayPtrArray(n);
	
	for (int i = 0; i < n; i++)
	{
		ioref node = oap->getOutputRefnum(i);
		char* name = oap->getOutputNodeName(node);
		int sz = ((i == 0) ? 3 : 2);
		storage_groupPtr p = first;
		int offset = 0;
		long mintime = (*p)[0].time;
		for (int i = 0; i < current_index; i++)
		{
			if ((*p)[i-offset].node == node)
				sz++;
			if (i-offset > storage_group::group_size)
			{
				offset += storage_group::group_size;
				p = p->next;
			}
		}
		MidiMsgArrayPtr mmap = new MidiMsgArray(sz);
		(*mmap)[0].set_meta(meta::txt_seq_or_track, name, strlen(name));
		(*mmap)[0].node = node;
		(*mmap)[0].time = 0;
		if (i==0) // put tempo in first track
		{
			uchar mybytes[3];
			mybytes[0] = // I'm very confused here
			(*mmap)[1].set_meta(meta::tempo, 
		// not finished yet
*/

// Author: Patrick Pelletier

		