/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

 /* This file gives example of how to create a parser with hooks into your own MIDI input 
processing functions.  In this case, a delayed octave generator is made. */

#include "octave_parser.h"
#include "data_structs.h"
#include "ParserRegistration.h"

int octave_parser::foo = 0;

static OMS_app *oa;
static scheduler* local_sched;
static int ou_node;
static pool<MIDI_msg> b(50, "");
static bool stop[128];

static const int note_len = 250, delay = 150;

static long wake( void *arg )
{
	MIDI_msg* m = (MIDI_msg*) arg;
	if (stop[m->key()])
	{
		b.release(m);
		return NEVER;
	}
	else
	{
		local_sched->play_note( *m, note_len );
		return delay;
	}
}

static
void
note_on( MIDI_msg *m ) 
{
	m->node = ou_node;
	if (m->vel())
	{
		local_sched->play_note( *m, note_len );
		
		stop[m->key()] = 0;

		MIDI_msg *n = b.allocate();
		if (!n)
			ew.w( "allocate failed" );
		else
		{
			*n = *m;

			task_data td( wake, n, m->time + delay );
			int e = local_sched->insert( td );
			if (e) ew.w( "insert failed in octave parser" );
		}
	}
	else stop[m->key()] = 1;
}

octave_parser::
octave_parser( OMS_app *in_oa, scheduler* in_sched )
{
	oa = in_oa;
	local_sched = in_sched;
	ou_node = oa->output_refnum("some_output");
	oa->open_input("some_input");

	msg_hnd[MIDI_msg::note_on] = note_on;
		
	MIDI_msg m;
	m.set( MIDI_msg::program_chn, 0 /* chan */, 20, 0 /* not used */, ou_node );
	//oa->write(&m);
}

static parser* newFunc()
{
	return (new octave_parser(oap, schedp));
}

static ParserRegistration reg(newFunc, "octave_parser");

// Author: Ben Denckla
